@@ -1,9 +1,12 @@
+from typing import List
+
 import sqlalchemy
 from sqlalchemy import inspect, text
 from sqlalchemy.ext.declarative import base
 from sqlalchemy.engine.reflection import Inspector
 from sqlalchemy.ext.hybrid import hybrid_property
 from sqlalchemy.orm import object_mapper, session, relationships, RelationshipProperty
+from sqlalchemy.orm.attributes import InstrumentedAttribute
 
 import logic_bank
 from logic_bank.rule_bank.rule_bank import RuleBank
@@ -15,8 +18,8 @@ from logic_bank.rule_type.constraint import Constraint
 from logic_bank.rule_type.formula import Formula
 from logic_bank.rule_type.parent_cascade import ParentCascade, ParentCascadeAction
 from logic_bank.rule_type.parent_check import ParentCheck
-from logic_bank.rule_type.row_event import EarlyRowEvent
-from logic_bank.util import ConstraintException
+from logic_bank.rule_type.row_event import EarlyRowEvent, RowEvent
+from logic_bank.util import ConstraintException, DotDict
 
 
 class LogicRow:
@@ -53,7 +56,7 @@ class LogicRow:
 
         self.row_sets = row_sets
         if row_sets is not None:  # eg, for debug as in upd_order_shipped test
-            row_sets.add_processed(logic_row=self)
+            row_sets.add_processed(logic_row=self)  # used in commit logic
 
         rb = RuleBank()
         self.rb = rb
@@ -161,21 +164,25 @@ class LogicRow:
         output = output.replace("]:", "] {" + msg + "}", 1)
         logic_bank.engine_logger.debug(output)
 
+    def new_logic_row(self, new_row_class: sqlalchemy.ext.declarative.api.DeclarativeMeta) -> 'LogicRow':
+        """ creates a new row of type new_row_class """
+        new_row = new_row_class()
+        result = LogicRow(row=new_row,
+                          old_row=new_row,
+                          ins_upd_dlt="ins",
+                          nest_level=self.nest_level + 1,
+                          a_session=self.session,
+                          row_sets=self.row_sets)
+        return result
+
     def make_copy(self, a_row: base) -> base:
         """
-        returns copy of row, or None
+        returns DotDict copy of row, or None
 
         :param a_row:
         :return:
         """
 
-        class DotDict(dict):
-            """dot.notation access to dictionary attributes"""
-            # thanks: https://stackoverflow.com/questions/2352181/how-to-use-a-dot-to-access-members-of-dictionary/28463329
-            __getattr__ = dict.get
-            __setattr__ = dict.__setitem__
-            __delattr__ = dict.__delitem__
-
         result = None
         if a_row is not None:
             result = DotDict({})
@@ -219,13 +226,26 @@ class LogicRow:
 
     def early_row_events(self):
         self.log_engine("early_events")
+        """
         early_row_events = rule_bank_withdraw.generic_rules_of_class(EarlyRowEvent)  # FIXME - review duplication
         for each_row_event in early_row_events:
             each_row_event.execute(self)
+        """
         early_row_events = rule_bank_withdraw.rules_of_class(self, EarlyRowEvent)
         for each_row_event in early_row_events:
             each_row_event.execute(self)
 
+    def row_events(self):
+        self.log_engine("early_events")
+        """
+        early_row_events = rule_bank_withdraw.generic_rules_of_class(EarlyRowEvent)  # FIXME - review duplication
+        for each_row_event in early_row_events:
+            each_row_event.execute(self)
+        """
+        row_events = rule_bank_withdraw.rules_of_class(self, RowEvent)
+        for each_row_event in row_events:
+            each_row_event.execute(self)
+
     def copy_rules(self):
         """ runs copy rules (get parent values) """
         copy_rules = rule_bank_withdraw.copy_rules(self)
@@ -247,6 +267,7 @@ class LogicRow:
         return role_def
 
     def link(self, to_parent: 'LogicRow'):
+        """ set self.to_parent (parent_accessor) = to_parent """
         parent_mapper = object_mapper(to_parent.row)
         parents_relationships = parent_mapper.relationships
         parent_role_name = None
@@ -363,6 +384,36 @@ class LogicRow:
                 return True
         return False
 
+    def are_attributes_changed(self, attr_list: List[InstrumentedAttribute]):
+        """ returns list of changed attr names, e.g.,
+
+        if not logic_row.are_attributes_changed([Employee.Salary, Employee.Title])
+        """
+        changes = []
+        for each_attr in attr_list:
+            if getattr(self.row, each_attr.key) != getattr(self.old_row, each_attr.key):
+                changes.append(each_attr.key)
+        return changes
+
+    def set_same_named_attributes(self, from_logic_row: 'LogicRow'):
+        """ copy like-named values from from_logic_row -> self """
+        row_mapper = object_mapper(self.row)
+        if self.row.__tablename__ == "Customerxx":
+            print("Debug Stop here")
+        from_attrs = object_mapper(from_logic_row.row).column_attrs
+        for each_attr in row_mapper.column_attrs:  # avoid parent objects, child collections
+            is_hybrid = isinstance(each_attr, hybrid_property)
+            each_attr_name = self.get_attr_name(mapper=row_mapper, attr=each_attr)
+            if each_attr_name is None:  # parent or child-list
+                raise Exception("attr_name is None, should not occur for row_mapper.column_attrs")
+            else:
+                if each_attr_name in self.table_meta.primary_key.columns.keys():
+                    debug_skip_primary_key_columns = True
+                else:
+                    if each_attr_name in from_attrs:
+                        setattr(self.row, each_attr_name, getattr(from_logic_row.row, each_attr_name))
+        return
+
     def get_old_child_rows(self, relationship: RelationshipProperty):
         """
         result = getattr(self.old_row, role_name)  # even with util.use_transient, yields nothing, unsure why
@@ -627,6 +678,12 @@ class LogicRow:
                                     ins_upd_dlt=ins_upd_dlt)
         return result_logic_row
 
+    def early_row_event_all_classes(self, verb_reason: str):
+        rules_bank = RuleBank()
+        if rules_bank._early_row_event_all_classes is not None:
+            self.log("early_row_event_all_classes - " + verb_reason)
+            rules_bank._early_row_event_all_classes(self)
+
     def update(self, reason: str = None, row: base = None):
         """
         make updates - with logic - in events, for example
@@ -634,7 +691,7 @@ class LogicRow:
         row = sqlalchemy read
         logic_row.update(row=row, msg="my log message")
         """
-        if row is not None:
+        if row is not None:  # FIXME why??
             user_logic_row = self.user_row_update(row=row, ins_upd_dlt="upd")
             user_logic_row.update(reason=reason)
         else:
@@ -648,21 +705,23 @@ class LogicRow:
             self.constraints()
             self.parent_cascade_attribute_changes_to_children()  # child chaining (cascade changed parent references)
             self.parent_cascade_pk_change()  # actions - delete, nullify, prevent
+            self.row_events()
             if self.row_sets is not None:  # eg, for debug as in upd_order_shipped test
                 self.row_sets.remove_submitted(logic_row=self)
 
     def insert(self, reason: str = None, row: base = None):
         """
-        make updates - with logic - in events, for example
+        make inserts - with logic - in events, for example
 
-        row = sqlalchemy read
-        logic_row.update(row=row, msg="my log message")
+        row = mapped_class()
+        logic_row.insert(row=row, msg="my log message")
         """
         if row is not None:
             user_logic_row = self.user_row_update(row=row, ins_upd_dlt="ins")
             user_logic_row.insert(reason=reason)
         else:
             self.reason = reason
+            self.early_row_event_all_classes("Insert - " + reason)
             self.log("Insert - " + reason)
             self.load_parents_on_insert()
             self.early_row_events()
@@ -670,7 +729,8 @@ class LogicRow:
             self.formula_rules()
             self.adjust_parent_aggregates()
             self.constraints()
-            # self.cascade_to_children()
+            if self.row_sets is not None:  # eg, for debug as in upd_order_shipped test
+                self.row_sets.remove_submitted(logic_row=self)
 
     def delete(self, reason: str = None, row: base = None):
         """
